
source("modules/data_load/dates2load.R")

# Set params -----------------------------------------
hcst.inityear <- recipe$Analysis$Time$hcst_start
hcst.endyear <- recipe$Analysis$Time$hcst_end
ltmin <- recipe$Analysis$Time$leadtimemin
ltmax <- recipe$Analysis$Time$leadtimemax
lats.min <- recipe$Analysis$Region$Regional[[1]]$latmin
lats.max <- recipe$Analysis$Region$Regional[[1]]$latmax
lons.min <- recipe$Analysis$Region$Regional[[1]]$lonmin
lons.max <- recipe$Analysis$Region$Regional[[1]]$lonmax
ref.name <- recipe$Analysis$Datasets$Reference$name
systems <- recipe$Analysis$Datasets$System

stream <- verifications$stream
sdates <- verifications$fcst.sdate

## TODO: define fcst.name
##fcst.name <- recipe$Analysis$Datasets$System[[sys]]$name

# get sdates array
sdates <- dates2load(recipe, logger)

# get esarchive datasets dict:
archive <- read_yaml(paste0(recipe$Run$code_dir,"conf/archive.yml"))$archive
dataset_descrip <- archive[[fcst.name]]

freq.hcst <- unlist(dataset_descrip[[store.freq]][variable])
reference_descrip <- archive[[ref.name]]
freq.obs <- unlist(reference_descrip[[store.freq]][variable])
obs.dir <- reference_descrip$src
fcst.dir <- dataset_descrip$src 
hcst.dir <- dataset_descrip$src
fcst.nmember <- dataset_descrip$nmember$fcst
hcst.nmember <- dataset_descrip$nmember$hcst

if ("accum" %in% names(reference_descrip)) {
  accum <- unlist(reference_descrip$accum[store.freq][[1]])
} else {
  accum <- FALSE
}

# -----------
obs.path <- paste0("/esarchive/",
            obs.dir, store.freq, "/$var$",
            freq.obs,"$var$_$file_date$.nc")

hcst.path <- paste0("/esarchive/",
             hcst.dir, store.freq, "/$var$",
             freq.hcst,"$var$_$file_date$01",
             ".nc")

# Regrid:
#-------------------------------------------------------------------
regrid_params <- get_regrid_params(recipe)

# Timeseries load 
#-------------------------------------------------------------------

hcst <- Start(dat = hcst.path,
              var = variable,
              file_date = file_dates,
              time = indices(ltmin:ltmax),
              latitude = values(list(lats.min, lats.max)),
              latitude_reorder = Sort(decreasing =  
                                      dataset_descrip$lat_decreasing_sort),
              longitude = values(list(lons.min, lons.max)),
              longitude_reorder = CircularSort(
                          dataset_descrip$lon_circular_sort$ini,
                          dataset_descrip$lon_circular_sort$end),
              transform = fcst.transform,
              transform_params = list(grid = fcst.gridtype,
                                      method = fcst.gridmethod,
                                      crop = c(lons.min, lons.max,
                                               lats.min, lats.max)),
              transform_vars = c('latitude', 'longitude'),
              synonims = list(latitude = c('lat','latitude'),
                              longitude = c('lon','longitude'),
                              member = c('ensemble')), 
              member = indices(1:hcst.nmember),
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = 'file_date'),
              split_multiselected_dims = TRUE,
              retrieve = TRUE)

hcst.NA_files <- c(attributes(hcst)$NotFoundFiles)
hcst.NA_files <- hcst.NA_files[!is.na(hcst.NA_files)]
try(hcst.NA_files <- hcst.NA_files[order(hcst.NA_files)])

dates <- attr(hcst, 'Variables')$common$time
dates_file <- sapply(dates, format, '%Y%m%d')
dates_file <- format(as.Date(dates_file, '%Y%m%d'), "%Y%m")
dim(dates_file) <- dim(Subset(hcst, 
                              along=c('dat','var',
                              'latitude', 'longitude', 'member'), 
                              list(1,1,1,1,1), drop="selected")) 

obs <- Start(dat = obs.path,		  
             var = variable,
	       file_date = dates_file,
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(decreasing =  
                                     reference_descrip$lat_decreasing_sort),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = CircularSort(
                          reference_descrip$lon_circular_sort$ini,
                          reference_descrip$lon_circular_sort$end),
             synonims = list(latitude = c('lat','latitude'),
                             longitude = c('lon','longitude')), 
             transform = obs.transform,
      transform_params = list(grid = obs.gridtype,
                       method = obs.gridmethod,
                       crop = c(lons.min, lons.max,
                                              lats.min, lats.max)), 
      transform_vars = c('latitude', 'longitude'),
	       return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = 'file_date'),
	       split_multiselected_dims = TRUE,
	       retrieve = TRUE)

dates_file <- paste0(dates_file, '01')
dim(dates_file) <- dim(Subset(hcst,
                              along=c('dat','var',
                              'latitude', 'longitude', 'member'),
                              list(1,1,1,1,1), drop="selected"))

file_dates <- paste0(file_dates, '01')
dim(file_dates) <- dim(Subset(hcst,
                              along=c('dat','var',
                              'latitude', 'longitude', 'member', 'time'),
                              list(1,1,1,1,1,1), drop="selected"))



obs.NA_dates.ind <- Apply(obs, 
                          fun=(function(x){ all(is.na(x))}), 
                          target_dims=c('time', 'latitude', 'longitude'))[[1]]
obs.NA_dates <- dates_file[obs.NA_dates.ind]
obs.NA_dates <- obs.NA_dates[order(obs.NA_dates)]
obs.NA_files <- paste0(obs.dir, store.freq,"/",variable,"_",
                       freq.obs,"obs.grid","/",variable,"_",obs.NA_dates,".nc")

if (any(is.na(hcst))){
  fatal(logger,
    paste(" ERROR: MISSING HCST VALUES FOUND DURING LOADING # ",
          " ################################################# ",
          " ######           MISSING FILES               #### ",
          " ################################################# ",
          "hcst files:",
          hcst.NA_files,
          " ################################################# ",
          " ################################################# ",
          sep="\n"))
  quit(status = 1)
}

if (any(is.na(obs)) && !identical(obs.NA_dates,character(0))){
  fatal(logger,
    paste(" ERROR: MISSING OBS VALUES FOUND DURING  LOADING # ",
          " ################################################# ",
          " ######           MISSING FILES               #### ",
          " ################################################# ",
          "obs files:",
          obs.NA_files,
          " ################################################# ",
          " ################################################# ",
          sep="\n"))
  quit(status=1)
}

info(logger,
     "######### DATA LOADING COMPLETED SUCCESFULLY ##############")

default_dims <- c(dat = 1, var = 1, sweek = 1, 
                  sday = 1, syear = 1, time = 1, 
                  latitude = 1, longitude = 1, member = 1)

default_dims[names(dim(obs))] <- dim(obs)
dim(obs) <- default_dims

lon <- attr(obs, 'Variables')$dat1$longitude
lat <- attr(obs, 'Variables')$dat1$latitude
hcst.times <- attr(hcst, 'Variables')$common$time
hcst.times <- sort(unique(sapply(as.character(hcst.times),
                   substr, 1, 10)))

#obs<-Subset(obs,c('dat','var'),list(1,1),drop='selected')
#hcst<-Subset(hcst,c('dat','var'),list(1,1),drop='selected')
#if (stream == "fcst"){ 
#  fcst<-Subset(fcst,c('dat','var'),list(1,1),drop='selected')
#}

#filters negative values in accum vars
if (accum){
   obs[obs < 0 ] <- 0
   hcst[hcst < 0 ] <- 0
}

sdates.hcst <- file_dates
leadtimes.hcst <- dates_file

